@@ -106,7 +106,7 @@ init_threads (cherokee_bench_t *bench)
 	{
 		/* Construct the object
 		 */
-		ret = cherokee_bench_thread_new (&thread, bench, fd_limit/thread_number);
+		ret = cherokee_bench_thread_new (&thread, bench, fd_limit, fd_limit/thread_number);
 		if (ret < ret_ok) return ret;
 
 		/* Configure the object
