@@ -1,6 +1,7 @@
 from CherokeeTest import *
 
-list = ["Test01", "TestRangeFrom", "TestRangeTo", "TestRangeFromTo", "TestOutOfRangeOverflow", "TestOutOfRangeSwitched"]
+list = ["Test01", "TestRangeFrom", "TestRangeTo", "TestRangeFromTo",
+        "TestOutOfRangeOverflow", "TestOutOfRangeSwitched", "TestErrorDocumentRedirect"]
 
 
 def random_string (length):
@@ -27,7 +28,7 @@ class Test01 (CherokeeTest):
 
         self.__name__    = "data consistency"
         self.__port__    = 1111
-        self.__content__ = encodestring(random_string(randint(2000,10000)))
+        self.__content__ = encodestring(random_string(randint(2000,100000)))
 
     def AnalizeResult (self):
         import httplib
@@ -92,12 +93,13 @@ class TestRange (Test01):
 
         data = response.read()
         if data != self.__content__[self.__range_start__:self.__range_end__]:            
+            if data == self.__content__:
+                print "Wrong response data: Web server didn't use the range"
+                return True
+
             len_data = len(data)
             len_cont = len(self.__content__)
-            if len_data == len_cont:
-                print "Wrong response data: Web server didn't use the range"
-            else:
-                print "Wrong response data: %d instead if %d" %(len_data, len(self.__content__[self.__range_start__:self.__range_end__]))
+            print "Wrong response data: %d instead if %d" % (len_data, len(self.__content__[self.__range_start__:self.__range_end__]))
             return True
 
         conn.close()
@@ -181,7 +183,7 @@ class TestOutOfRangeOverflow (TestOutOfRange):
         return TestOutOfRange.AnalizeResult (self)
 
 
-class TestOutOfRangeSwitched  (TestOutOfRange):
+class TestOutOfRangeSwitched (TestOutOfRange):
     def __init__ (self):
         TestOutOfRange.__init__ (self)
         self.__name__ = "switched range detection"
@@ -194,3 +196,34 @@ class TestOutOfRangeSwitched  (TestOutOfRange):
         self.__range_end__   = min(r1,r2)
         return TestOutOfRange.AnalizeResult (self)
 
+
+class TestErrorDocumentRedirect (CherokeeTest):
+    def __init__ (self):
+        CherokeeTest.__init__(self)
+
+        self.__port__    = 1111
+        self.__name__    = "ErrorDocument 404"
+
+    def AnalizeResult (self):
+        import httplib
+
+        conn = httplib.HTTPConnection("localhost:%s" % (self.__port__))
+        conn.request("GET", "/this_doesnt_exist")
+
+        response = conn.getresponse()
+        if response.status not in (301, 302):
+            print "Wrong response:", response.status
+            return True
+
+        conn.close()
+        return False
+
+    def GenerateConfig (self):        
+        tmpdir = self.MakeTempDir()
+
+        return """
+        Port %s
+        DocumentRoot %s/
+        ErrorDocument 404 http://localhost:%s/error404.html
+        Directory / { Handler file }
+        """ % (self.__port__, tmpdir, self.__port__)
