@@ -37,23 +37,23 @@
 # include <arpa/inet.h>  
 #endif
 
-#if defined(HAVE_IPV6) && defined(__FreeBSD__)
-# define s6_addr32 __u6_addr.__u6_addr32
-#endif
-
 #include "access.h"
 
+#ifndef AF_INET6
+# define AF_INET6 10
+#endif
+
 
 typedef enum { 
-	ipv4 = AF_INET, 
+	ipv4 = AF_INET,
 	ipv6 = AF_INET6
 } ip_type_t;
 
 
 typedef union {
-                struct in_addr  ip4;
+	struct in_addr  ip4;
 #ifdef HAVE_IPV6
-                struct in6_addr ip6;
+	struct in6_addr ip6;
 #endif
 } ip_t;
 
@@ -73,25 +73,40 @@ typedef struct {
 #define SUBNET_NODE(x) ((subnet_item_t *)(x))
 
 
-ip_item_t *
+
+
+static ip_item_t *
 new_ip (void) 
 {
 	ip_item_t *n = malloc (sizeof(ip_item_t));
 	if (n == NULL) return NULL;
+
+	printf ("ip %d\n", sizeof(ip_item_t));
+
 	INIT_LIST_HEAD((list_t*)n);
+
 	return n;
 }
 
-subnet_item_t *
+static ret_t
+free_ip (ip_item_t *ip)
+{
+	free (ip);
+	return ret_ok;
+}
+
+static subnet_item_t *
 new_subnet (void) 
 {
 	subnet_item_t *n = malloc (sizeof(subnet_item_t));
 	if (n == NULL) return NULL;
+
 	INIT_LIST_HEAD((list_t*)n);
 	return n;
 }
 
 
+
 ret_t 
 cherokee_access_new (cherokee_access_t **entry)
 {
@@ -123,7 +138,7 @@ print_ip (ip_type_t type, ip_t *ip)
 		return;
 	}
 #else
-	printf ("%s", inet_ntoa (ip.ipv4));
+	printf ("%s", inet_ntoa (ip->ipv4));
 #endif
 }
 
@@ -162,12 +177,18 @@ parse_ip (char *ip, ip_item_t *n)
 	n->type = ((index (ip, ':') != NULL) || 
 		   (strlen(ip) > 15)) ? ipv6 : ipv4;
 	
+#ifndef HAVE_IPV6
+	if (n->type == ipv6) {
+		return ret_error;
+	}
+#endif
+
 	/* Parse the IP string
 	 */
 #ifdef HAVE_INET_PTON
 	ok = (inet_pton (n->type, ip, &n->ip) > 0);
 
-#ifdef HAVE_IPV6
+# ifdef HAVE_IPV6
 	if (n->type == ipv6) {
 		if (IN6_IS_ADDR_V4MAPPED (&(n->ip).ip6)) {
 			PRINT_ERROR ("This IP '%s' is IPv6-mapped IPv6 address.  "
@@ -176,7 +197,7 @@ parse_ip (char *ip, ip_item_t *n)
 			return ret_error;
 		}
 	}
-#endif /* HAVE_IPV6 */
+# endif /* HAVE_IPV6 */
 
 #else
 	ok = (inet_aton (ip, &n->ip) != 0);
@@ -235,10 +256,11 @@ parse_netmask (char *netmask, subnet_item_t *n)
 	/* Special case
 	 */
 	if (num == 128) {
-		n->mask.ip6.s6_addr32[0] = ~0L;
-		n->mask.ip6.s6_addr32[1] = ~0L;
-		n->mask.ip6.s6_addr32[2] = ~0L;
-		n->mask.ip6.s6_addr32[3] = ~0L;
+		int i;
+
+		for (i=0; i<16; i++) {
+			n->mask.ip6.s6_addr[i] = 0;
+		}
 
 		return ret_ok;
 	}
@@ -279,13 +301,16 @@ cherokee_access_add_ip (cherokee_access_t *entry, char *ip)
 	n = new_ip();
 	if (n == NULL) return ret_error;
 
-	list_add ((list_t *)n, &entry->list_ips);
-
 	ret = parse_ip (ip, n);
 	if (ret < ret_ok) {
 		PRINT_ERROR ("IP address '%s' seems to be invalid\n", ip);
+
+		free_ip(n);
+		return ret;
 	}
 
+	list_add ((list_t *)n, &entry->list_ips);
+
 	return ret;
 }
 
