@@ -174,15 +174,22 @@ cherokee_buffer_make_empty (cherokee_buffer_t *buf)
 ret_t
 cherokee_buffer_move_to_begin (cherokee_buffer_t *buf, int pos)
 {
-	return_if_fail (pos <= buf->size, ret_error);
+	int offset;
 
+	return_if_fail (pos <= buf->size, ret_error);
 	if (pos <= 0) return ret_ok;
 
-	if (pos <= buf->len) {
-		memmove (buf->buf, buf->buf+pos, buf->len-pos+1);
-		buf->len -= pos;
+	offset = MIN(pos, buf->len);
+	memmove (buf->buf, buf->buf+offset, (buf->len - offset) +1);
+	buf->len -= offset;
+
+#if 0
+	if (strlen(buf->buf) != buf->len) {
+		PRINT_ERROR ("ERROR: cherokee_buffer_move_to_begin(): strlen=%d buf->len=%d\n", 
+			     strlen(buf->buf), buf->len);
 	}
-	   
+#endif
+
 	return ret_ok;
 }
 
