@@ -179,3 +179,33 @@ get_system_fd_num_limit (int *limit)
 	return ret_ok;
 }
 
+
+
+#ifndef HAVE_STRSEP
+char *
+strsep (char** str, const char* delims)
+{
+    char* token;
+
+    if (*str == NULL) {
+        /* No more tokens 
+	 */
+        return NULL;
+    }
+
+    token = *str;
+    while (**str!='\0') {
+        if (strchr(delims,**str)!=NULL) {
+            **str='\0';
+            (*str)++;
+            return token;
+        }
+        (*str)++;
+    }
+
+    /* There is no other token 
+     */
+    *str=NULL;
+    return token;
+}
+#endif
