@@ -141,6 +141,14 @@ typedef enum {
         const unsigned int obj ## _size = size;  \
 	char obj[size]
 
+#ifndef MIN
+# define MIN(x,y) ((x<y) ? x : y)
+#endif
+
+#ifndef MAX
+# define MAX(x,y) ((x>y) ? x : y)
+#endif
+
 
 #ifdef HAVE_PTHREAD
 # define CHEROKEE_MUTEX_LOCK(m)     pthread_mutex_lock(m)
@@ -205,4 +213,8 @@ char *min_string_not_null (char *s1, char *s2);
 
 ret_t get_system_fd_num_limit (int *limit);
 
+#ifndef HAVE_STRSEP
+char *strsep (char** str, const char* delims);
+#endif
+
 #endif /* __CHEROKEE_COMMON_H__ */
