@@ -66,7 +66,7 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	n->range_start    = 0;
 	n->range_end      = 0;
 	n->vserver        = NULL;
-	n->log_at_end     = 0;
+	n->log_at_end     = 1;
 	n->realm_ref      = NULL;
 	n->mmaped         = NULL;
 	n->thread         = NULL;
@@ -157,7 +157,7 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 	cnt->range_end    = 0;
 	cnt->logger_ref   = NULL;
 	cnt->tcp_cork     = 0;
-	cnt->log_at_end   = 0;
+	cnt->log_at_end   = 1;
 	cnt->realm_ref    = NULL;
 	cnt->post_len     = 0;
 	cnt->mmaped       = NULL;
@@ -231,42 +231,6 @@ cherokee_connection_mrproper (cherokee_connection_t *cnt)
 }
 
 
-static inline void
-add_error_code_string_to_buffer (cherokee_connection_t *cnt)
-{
-	switch(cnt->error_code) {
-	case http_ok:
-		cherokee_buffer_add (cnt->buffer, http_ok_string, 6); break;
-	case http_accepted:
-		cherokee_buffer_add (cnt->buffer, http_accepted_string CRLF, 12); break;
-	case http_partial_content:
-		cherokee_buffer_add (cnt->buffer, http_partial_content_string CRLF, 19); break;		
-	case http_bad_request:
-		cherokee_buffer_add (cnt->buffer, http_bad_request_string CRLF, 15); break;
-	case http_access_denied:
-		cherokee_buffer_add (cnt->buffer, http_access_denied_string CRLF, 13); break;
-	case http_not_found:
-		cherokee_buffer_add (cnt->buffer, http_not_found_string CRLF, 13); break;
-	case http_internal_error:
-		cherokee_buffer_add (cnt->buffer, http_internal_error_string CRLF, 25); break;
-	case http_moved_permanently: 
-		cherokee_buffer_add (cnt->buffer, http_moved_permanently_string CRLF, 21); break;
-	case http_moved_temporarily:
-		cherokee_buffer_add (cnt->buffer, http_moved_temporarily_string CRLF, 21); break;	
-	case http_unauthorized:
-		cherokee_buffer_add (cnt->buffer, http_unauthorized_string CRLF, 26); break;
-	case http_not_modified:
-		cherokee_buffer_add (cnt->buffer, http_not_modified_string CRLF, 16); break;	
-	case http_length_required:
-		cherokee_buffer_add (cnt->buffer, http_length_required_string CRLF, 21); break;	
-	case http_range_not_satisfiable:
-		cherokee_buffer_add (cnt->buffer, http_range_not_satisfiable_string CRLF, 37); break;
-	default:
-		SHOULDNT_HAPPEN;
-	}
-}
-
-
 static ret_t
 send_buffer_unsafe (cherokee_connection_t *cnt)
 {
@@ -359,9 +323,8 @@ build_response_header (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
 		break;
 	}
 	
-	add_error_code_string_to_buffer (cnt);
-	cherokee_buffer_add (buffer, CRLF, 2); 
-
+	cherokee_http_code_copy (cnt->error_code, buffer);
+	cherokee_buffer_add (buffer, CRLF, 2);
 
 	/* Add server name
 	 */
@@ -486,9 +449,15 @@ cherokee_connection_send_header (cherokee_connection_t *cnt)
 	/* Send it
 	 */
 	ret = send_buffer_unsafe (cnt);
-
 	cherokee_buffer_make_empty  (cnt->buffer);
 
+#if 0
+	if (ret < ret_ok) {
+		PRINT_ERROR ("cherokee_connection_send_header ret=%d\n", ret);
+		return ret;
+	}
+#endif
+
 	return ret;
 }
 
@@ -565,7 +534,7 @@ ret_t
 cherokee_connection_recv (cherokee_connection_t *cnt, cherokee_buffer_t *buffer, uint32_t *len)
 {
 	ret_t   ret;
-	ssize_t readed;
+	ssize_t readed = 0;
 	
 	ret = cherokee_socket_read (cnt->socket, buffer, DEFAULT_RECV_SIZE, &readed);
 
@@ -656,10 +625,11 @@ cherokee_connection_set_cork (cherokee_connection_t *cnt, int enable)
 ret_t
 cherokee_connection_send (cherokee_connection_t *cnt)
 {
-	ret_t ret;
-	int   sent;
+	ret_t  ret;
+	size_t sent = 0;
 
 	ret = cherokee_socket_write (cnt->socket, cnt->buffer, &sent);
+	if (ret != ret_ok) return ret;
 
 	if (sent > 0) {
 		if (sent == cnt->buffer->len) {
@@ -812,8 +782,6 @@ get_encoding (cherokee_connection_t    *cnt,
 		*i2 = tmp;    /* (2') */
 
 		if (cnt->encoder != NULL) {
-			printf ("hay encoder\n");
-
 			/* Init the encoder related objects
 			 */
 			ret = cherokee_encoder_init (cnt->encoder, cnt);
@@ -1344,73 +1312,95 @@ cherokee_connection_open_request (cherokee_connection_t *cnt)
 }
 
 
+static ret_t
+build_hardcoded_response_page (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
+{		
+	cherokee_buffer_add (cnt->buffer, "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">" CRLF, 50);
+	   
+	/* Add page title
+	 */
+	cherokee_buffer_add (cnt->buffer, "<html><head><title>", 19);
+	cherokee_http_code_copy (cnt->error_code, cnt->buffer);
+
+	/* Add big banner
+	 */
+	cherokee_buffer_add (cnt->buffer, "</title></head><body><h1>", 25);
+	cherokee_http_code_copy (cnt->error_code, cnt->buffer);
+	cherokee_buffer_add (cnt->buffer, "</h1>", 5);
+
+	/* Maybe add some info
+	 */
+	switch (cnt->error_code) {
+	case http_not_found:
+		if (cnt->request) {
+			cherokee_buffer_swap_chars (cnt->request, '<', ' ');
+			cherokee_buffer_swap_chars (cnt->request, '>', ' ');
+
+			cherokee_buffer_add (cnt->buffer, "The requested URL ", 18);
+			cherokee_buffer_add_buffer (cnt->buffer, cnt->request);
+			cherokee_buffer_add (cnt->buffer, " was not found on this server.", 30);
+		}
+		break;
+	case http_bad_request:
+		cherokee_buffer_add (cnt->buffer, 
+				     "Your browser sent a request that this server could not understand.", 66);
+		break;
+	case http_moved_permanently:
+		cherokee_buffer_add (cnt->buffer, 
+				     "The document has moved", 22);
+		break;
+	case http_moved_temporarily:
+		cherokee_buffer_add_va (cnt->buffer, 
+					"The document has moved <A HREF=\"%s\">here</A>.",
+					cnt->redirect->buf);
+		break;
+	case http_unauthorized:
+		cherokee_buffer_add (cnt->buffer, 
+				     "This server could not verify that you are authorized to access the document "
+				     "requested.  Either you supplied the wrong credentials (e.g., bad password), "
+				     "or your browser doesn't understand how to supply the credentials required", 225);
+	default:
+		break;
+	}
+	   
+	/* Add page foot
+	 */
+	if (CONN_SRV(cnt)->hideservername == 0) {
+		cherokee_buffer_add (cnt->buffer, "<p><hr>", 7); 
+
+		if (CONN_SRV(cnt)->hideversion) {
+			cherokee_buffer_add_version (cnt->buffer, CONN_SRV(cnt)->port, ver_port_html);
+		} else {
+			cherokee_buffer_add_version (cnt->buffer, CONN_SRV(cnt)->port, ver_full_html);
+		}
+	}
+  
+	cherokee_buffer_add (cnt->buffer, "</body></html>", 14); 
+
+	return ret_ok;
+}
+
+
 ret_t
 cherokee_connection_send_response_page_hardcoded (cherokee_connection_t *cnt)
 {
-	   ret_t  ret;
-	   
-	   cherokee_buffer_make_empty  (cnt->buffer);
-	   
-	   cherokee_buffer_add (cnt->buffer, "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">" CRLF, 50);
-	   
-	   /* Add page title
-	    */
-	   cherokee_buffer_add (cnt->buffer, "<html><head><title>", 19);
-	   add_error_code_string_to_buffer (cnt);
+	ret_t  ret;
+	
+	cherokee_buffer_make_empty (cnt->buffer);
 
-	   /* Add big banner
-	    */
-	   cherokee_buffer_add (cnt->buffer, "</title></head><body><h1>", 25);
-	   add_error_code_string_to_buffer (cnt);
-	   cherokee_buffer_add (cnt->buffer, "</h1>", 5);
+	ret = build_hardcoded_response_page (cnt, cnt->buffer);
+	if (ret < ret_ok) return ret;
 
-	   /* Maybe add some info
-	    */
-	   switch (cnt->error_code) {
-	   case http_not_found:
-		   if (cnt->request) {
-			   cherokee_buffer_swap_chars (cnt->request, '<', ' ');
-			   cherokee_buffer_swap_chars (cnt->request, '>', ' ');
-
-			   cherokee_buffer_add (cnt->buffer, "The requested URL ", 18);
-			   cherokee_buffer_add_buffer (cnt->buffer, cnt->request);
-			   cherokee_buffer_add (cnt->buffer, " was not found on this server.", 30);
-		   }
-		   break;
-	   case http_bad_request:
-		   cherokee_buffer_add (cnt->buffer, 
-					"Your browser sent a request that this server could not understand.", 66);
-		   break;
-	   case http_moved_permanently:
-		   cherokee_buffer_add (cnt->buffer, 
-					"The document has moved", 22);
-		   break;
-	   case http_unauthorized:
-		   cherokee_buffer_add (cnt->buffer, 
-					"This server could not verify that you are authorized to access the document "
-					"requested.  Either you supplied the wrong credentials (e.g., bad password), "
-					"or your browser doesn't understand how to supply the credentials required", 225);
-	   default:
-		   break;
-	   }
-	   
-	   /* Add page foot
-	    */
-	   if (CONN_SRV(cnt)->hideservername == 0) {
-		   cherokee_buffer_add (cnt->buffer, "<p><hr>", 7); 
-
-		   if (CONN_SRV(cnt)->hideversion) {
-			   cherokee_buffer_add_version (cnt->buffer, CONN_SRV(cnt)->port, ver_port_html);
-		   } else {
-			   cherokee_buffer_add_version (cnt->buffer, CONN_SRV(cnt)->port, ver_full_html);
-		   }
-	   }
-  
-	   cherokee_buffer_add (cnt->buffer, "</body></html>", 14); 
- 
-	   /* Send it
-	    */
-	   return send_buffer_unsafe (cnt);
+	/* Send it
+	 */
+	ret = send_buffer_unsafe (cnt);
+#if 0
+	if (ret < ret_ok) {
+		PRINT_ERROR ("cherokee_connection_send_response_page_hardcoded ret=%d\n", ret);
+		return ret;
+	}
+#endif
+	return ret;
 }
 
 
@@ -1438,6 +1428,7 @@ cherokee_connection_log_or_delay (cherokee_connection_t *conn)
 	if (conn->log_at_end == 0) {
 		if (conn->logger_ref != NULL) {
 			ret = cherokee_logger_write_access (conn->logger_ref, conn);
+			conn->log_at_end = 0;
 		}
 	}
 
@@ -1448,13 +1439,11 @@ cherokee_connection_log_or_delay (cherokee_connection_t *conn)
 ret_t 
 cherokee_connection_log_delayed (cherokee_connection_t *conn)
 {
-	ret_t ret = ret_ok;
+	ret_t ret;
 
-	if (conn->log_at_end) {
-		if (conn->logger_ref != NULL) {
-			ret = cherokee_logger_write_access (conn->logger_ref, conn);
-		}		
+	if ((conn->log_at_end) && (conn->logger_ref)) {
+		ret = cherokee_logger_write_access (conn->logger_ref, conn);
 	}
 
-	return ret_ok;
+	return ret;
 }
