@@ -91,17 +91,18 @@ cherokee_encoder_gzip_new (cherokee_encoder_gzip_t **encoder)
 {
 	CHEROKEE_NEW_STRUCT (n, encoder_gzip);
 
+	/* Init 	
+	 */
 	cherokee_encoder_init_base (ENCODER(n));
 
-	ENCODER(n)->free        = (encoder_func_free_t) cherokee_encoder_gzip_free;
+	MODULE(n)->free         = (encoder_func_free_t) cherokee_encoder_gzip_free;
+	MODULE(n)->init         = (encoder_func_encode_t) cherokee_encoder_gzip_init;
 	ENCODER(n)->add_headers = (encoder_func_add_headers_t) cherokee_encoder_gzip_add_headers;
-	ENCODER(n)->init        = (encoder_func_encode_t) cherokee_encoder_gzip_init;
 	ENCODER(n)->encode      = (encoder_func_encode_t) cherokee_encoder_gzip_encode;
 
 	/* Return the object
 	 */
 	*encoder = n;
-
 	return ret_ok;
 }
 
