@@ -57,9 +57,12 @@
 #undef  USE_POLL
 #undef  USE_EPOLL
 #undef  USE_SELECT
+#undef  USE_KQUEUE
+
 #define USE_SELECT
 
-# include <sys/select.h>
+// # include <sys/poll.h>
+// # include <sys/select.h>
 */
 /* </DEBUGGING>
  */
@@ -78,7 +81,9 @@ typedef enum {
 
 
 typedef struct {
-	int nfiles;
+	int nfiles;            /* Max. fds in this FD poll */
+	int system_nfiles;     /* Max. fds in the system   */
+	int npollfds;          /* Currently, how many FDs  */
 
 #ifdef USE_EPOLL
 	int                 ep_fd;
@@ -90,7 +95,7 @@ typedef struct {
 
 #ifdef USE_POLL
 	struct pollfd *pollfds;
-	int            npollfds;
+//	int            npollfds;
 	int           *fdidx;
 #endif
 
@@ -114,9 +119,9 @@ typedef struct {
 	int    *select_fds;
 	int    *select_fdidx;
 	int    *select_rfdidx;
-	int     nselect_fds;
 	int     maxfd;
 	int     maxfd_changed;
+//	int     nselect_fds;
 #endif
 
 #ifdef HAVE_PTHREAD
@@ -147,6 +152,7 @@ typedef struct {
 # define cherokee_fdpoll_check      fdpoll_poll_check
 # define cherokee_fdpoll_reset      fdpoll_poll_reset
 # define cherokee_fdpoll_get_method fdpoll_poll_get_method
+# define cherokee_fdpoll_is_full    fdpoll_poll_is_full
 #endif
 
 #ifdef USE_KQUEUE
@@ -174,7 +180,7 @@ typedef struct {
 #endif
 
 
-ret_t cherokee_fdpoll_new        (cherokee_fdpoll_t **fdp, int fd_limit);
+ret_t cherokee_fdpoll_new        (cherokee_fdpoll_t **fdp, int system_limit, int limit);
 ret_t cherokee_fdpoll_free       (cherokee_fdpoll_t  *fdp);
 ret_t cherokee_fdpoll_get_method (cherokee_fdpoll_t  *fdp, char **name);   
 
@@ -183,6 +189,6 @@ ret_t cherokee_fdpoll_del      (cherokee_fdpoll_t  *fdp, int fd);
 void  cherokee_fdpoll_set_mode (cherokee_fdpoll_t  *fdp, int fd, int rw);
 int   cherokee_fdpoll_check    (cherokee_fdpoll_t  *fdp, int fd, int rw);
 int   cherokee_fdpoll_watch    (cherokee_fdpoll_t  *fdp, int timeout_msecs);
-
+ret_t cherokee_fdpoll_is_full  (cherokee_fdpoll_t  *fdp);
 
 #endif /* __CHEROKEE_FD_POLL_H__ */
