@@ -32,16 +32,18 @@
 ret_t
 cherokee_handler_init_base (cherokee_handler_t *hdl, void *conn)
 {
-	/* Parent reference
+	/* Init the base class
 	 */
-	hdl->connection = conn;
+	cherokee_module_init_base (hdl);
 
 	/* Pure virtual methods
 	 */
-	hdl->init        = NULL;
-	hdl->free        = NULL;
 	hdl->step        = NULL;
 	hdl->add_headers = NULL;
+
+	/* Parent reference
+	 */
+	hdl->connection = conn;
 	
 	return ret_ok;
 }
@@ -53,8 +55,9 @@ cherokee_handler_init_base (cherokee_handler_t *hdl, void *conn)
 ret_t
 cherokee_handler_free (cherokee_handler_t *hdl)
 {
-	if (hdl->free)
-		return hdl->free (hdl);
+	if (MODULE(hdl)->free) {
+		return MODULE(hdl)->free (hdl); 
+	}
 	
 	return ret_error;
 }
@@ -63,8 +66,13 @@ cherokee_handler_free (cherokee_handler_t *hdl)
 inline ret_t
 cherokee_handler_init (cherokee_handler_t *hdl)
 {
-	if (hdl->init)
-		return hdl->init (hdl);
+	handler_func_init_t init_func;
+
+	init_func = (handler_func_init_t) MODULE(hdl)->init;
+
+	if (init_func) {
+		return init_func(hdl); 
+	}
 	
 	return ret_error;
 }
@@ -73,8 +81,9 @@ cherokee_handler_init (cherokee_handler_t *hdl)
 inline ret_t
 cherokee_handler_step (cherokee_handler_t *hdl, cherokee_buffer_t *buffer)
 {
-	if (hdl->step)
+	if (hdl->step) {
 		return hdl->step (hdl, buffer);
+	}
 	
 	return ret_error;
 }
@@ -83,8 +92,9 @@ cherokee_handler_step (cherokee_handler_t *hdl, cherokee_buffer_t *buffer)
 inline ret_t
 cherokee_handler_add_headers (cherokee_handler_t *hdl, cherokee_buffer_t *buffer)
 {
-	if (hdl->add_headers)
-		return hdl->add_headers (hdl, buffer);
+	if (hdl->add_headers) {
+		return hdl->add_headers (hdl, buffer); 
+	}
 
 	return ret_error;
 }
