@@ -27,11 +27,11 @@
 #define __CHEROKEE_HANDLER_H__
 
 #include "common.h"
+#include "module.h"
 #include "buffer.h"
 #include "table.h"
 #include "http.h"
 
-
 typedef enum {
 	hstat_unset,
 	hstat_sending,
@@ -58,16 +58,17 @@ typedef ret_t (* handler_func_add_headers_t) (void  *handler, cherokee_buffer_t
 
 
 typedef struct {
-	cherokee_handler_support_t support;
-
-	void *connection;
+	cherokee_module_t module;
 	
-	/* 	Pure virtual methods
+	/* Pure virtual methods
 	 */
-	handler_func_init_t         init;
-	handler_func_free_t         free;
 	handler_func_step_t         step;
 	handler_func_add_headers_t  add_headers;
+
+	/* Properties
+	 */
+	void *connection;
+	cherokee_handler_support_t support;
 	   
 } cherokee_handler_t;
 
