@@ -43,8 +43,8 @@
 /* hmm.. it will be wonderful if cherokee had a
  * cherokee_connection_get_mehod_name (or similar), and use it 
  * instead of this vector
- */
 static char *method[]  = {"GET", "POST", "HEAD", NULL};
+ */
 
 
 cherokee_module_info_t cherokee_cgi_info = {
@@ -75,15 +75,15 @@ cherokee_handler_cgi_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	HANDLER(n)->connection  = cnt;
 
-	HANDLER(n)->support = hsupport_complex_headers;
-
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_cgi_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_cgi_free;
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_cgi_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_cgi_free;
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_cgi_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_cgi_add_headers;
 
+	HANDLER(n)->connection  = cnt;
+	HANDLER(n)->support     = hsupport_complex_headers;
+
 	/* Process the request_string, and build the arguments table..
 	 * We'll need it later
 	 */
@@ -94,7 +94,7 @@ cherokee_handler_cgi_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t
 	n->pipeInput      = 0;
 	n->pipeOutput     = 0;
 	n->post_data_sent = 0;
-	n->pid            = 0;
+	n->pid            = -1;
 	n->pathInfo       = NULL;
 	n->fileName       = NULL;
 	n->data           = NULL;
@@ -110,8 +110,9 @@ cherokee_handler_cgi_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t
 static ret_t
 _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 {
-	char *p = NULL;
 	int   r;
+	char *p = NULL;
+	int   p_len;
 	cherokee_buffer_t *tmp;
 	char remote_ip[INET6_ADDRSTRLEN+1];
 	ret_t ret;
@@ -180,45 +181,28 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 	snprintf (temp, temp_size, "%d", CONN_SRV(conn)->port);
 	r += setenv ("SERVER_PORT", temp, 1);
 
-	switch (conn->header->version) {
-	case http_version_11:
-		r += setenv ("SERVER_PROTOCOL", "HTTP/1.1", 1);
-		break;
-	case http_version_10:
-		r += setenv ("SERVER_PROTOCOL", "HTTP/1.0", 1);
-		break;
-	default:
-		SHOULDNT_HAPPEN;
+	/* Set the HTTP version
+	 */
+	ret = cherokee_http_version_to_string (conn->header->version, &p, &p_len);
+	if (ret >= ret_ok) {
+		r += setenv ("SERVER_PROTOCOL", p, 1);
 	}
 
-	switch (conn->header->method) {
-	case http_get:
-		r += setenv ("REQUEST_METHOD", "GET", 1);		   
-		break;
-	case http_post:
-		r += setenv ("REQUEST_METHOD", "POST", 1);		   
-		break;
-	case http_head:
-		r += setenv ("REQUEST_METHOD", "HEAD", 1);		   
-		break;
-	default:
-		SHOULDNT_HAPPEN;
+	/* Set the method
+	 */
+	ret = cherokee_http_method_to_string (conn->header->method, &p, &p_len);
+	if (ret >= ret_ok) {
+		r += setenv ("REQUEST_METHOD", p, 1);
 	}
-
-	if (r != 0) {
-		return ret_error;
-	}
-
-	if (conn->header->method < 4) {
-		r = setenv ("REQUEST_METHOD", method[conn->header->method], 1);
-		if (r != 0) return ret_error;
-	}
-
-	if (!cherokee_buffer_is_empty (conn->host)) 
-	{
+	
+	/* Set the host name
+	 */
+	if (!cherokee_buffer_is_empty (conn->host)) {
 		p = index (conn->host->buf, ':');
 		if (p != NULL) *p = '\0';
+
 		r = setenv ("SERVER_NAME", conn->host->buf, 1);
+
 		if (p != NULL) *p = ':';
 		if (r != 0) return ret_error;
 	}
@@ -256,13 +240,13 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 }
 
 ret_t
-_extract_path(cherokee_handler_cgi_t* n)
+_extract_path (cherokee_handler_cgi_t *cgi)
 {
 	int dr_len;
 	char *cur;
 	cherokee_connection_t *conn;
 
-	conn = CONN(HANDLER(n)->connection);
+	conn = CONN(HANDLER(cgi)->connection);
 
 	/* Append URI to DocumentRoot 
 	 */
@@ -291,16 +275,16 @@ _extract_path(cherokee_handler_cgi_t* n)
 			{
 				/* This can be the CGI to run 
 				 */
-				cherokee_buffer_new (&n->pathInfo);
-				cherokee_buffer_new (&n->fileName);
+				cherokee_buffer_new (&cgi->pathInfo);
+				cherokee_buffer_new (&cgi->fileName);
 
-				cherokee_buffer_add_buffer (n->fileName, 
+				cherokee_buffer_add_buffer (cgi->fileName, 
 							    conn->local_directory);
 
 				/* Path info 
 				 */
 				*cur = '/';
-				cherokee_buffer_add (n->pathInfo, 
+				cherokee_buffer_add (cgi->pathInfo, 
 						     cur, strlen(cur));
 
 				break;
@@ -310,9 +294,9 @@ _extract_path(cherokee_handler_cgi_t* n)
 
 	/* Is the filename set? 
 	 */
-	if (n->fileName == NULL) {
-		cherokee_buffer_new (&n->fileName);
-		cherokee_buffer_add_buffer (n->fileName, conn->local_directory);
+	if (cgi->fileName == NULL) {
+		cherokee_buffer_new (&cgi->fileName);
+		cherokee_buffer_add_buffer (cgi->fileName, conn->local_directory);
 	}
 
 	/* Restore local_directory 
@@ -365,18 +349,18 @@ _send_post_data(cherokee_handler_cgi_t *cgi)
 
 
 ret_t 
-cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
+cherokee_handler_cgi_init (cherokee_handler_cgi_t *cgi)
 {
 	ret_t ret;
 	int   re;
 	int   pid;
 	int   pipesServer[2], pipesCGI[2];
-	cherokee_connection_t *conn = HANDLER_CONN(n);
+	cherokee_connection_t *conn = HANDLER_CONN(cgi);
 
 
 	/* Extracts PATH_INFO and filename from request uri 
 	 */
-	ret = _extract_path (n);
+	ret = _extract_path(cgi);
 	if (ret < ret_ok) { 
 		return ret;
 	}
@@ -401,7 +385,7 @@ cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
 
 		/* Sets the new environ. 
 		 */			
-		_set_envs_cgi (conn, n);
+		_set_envs_cgi (conn, cgi);
 
 		/* Change stdin and out 
 		 */
@@ -411,12 +395,12 @@ cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
 		dup2 (pipesServer[0], 0);  /* stdin  */
 		dup2 (pipesCGI[1], 1);     /* stdout */
 
-		_LOG("CGI: Executing %s %s\n", n->fileName->buf,
-		     (n->parameter)? n->parameter->buf : "");
+		_LOG("CGI: Executing %s %s\n", cgi->fileName->buf,
+		     (cgi->parameter)? cgi->parameter->buf : "");
 
 		/* Change the current directory to to the program's own directory.
 		 */
-		absolute_path = n->fileName->buf;
+		absolute_path = cgi->fileName->buf;
 		file = strrchr (absolute_path, '/');
 		*file = '\0';
 		chdir (absolute_path);
@@ -425,10 +409,10 @@ cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
 
 		/* Execute the new CGI 
 		 */
-		if (n->parameter == NULL) {
+		if (cgi->parameter == NULL) {
 			re = execl (absolute_path, file, NULL);
 		} else {
-			re = execl (absolute_path, file, n->parameter->buf, NULL);	
+			re = execl (absolute_path, file, cgi->parameter->buf, NULL);	
 		}
 
 		if (re < 0) {
@@ -451,71 +435,125 @@ cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
 	close (pipesServer[0]);
 	close (pipesCGI[1]);
 
-	n->pid        = pid;
-	n->pipeInput  = pipesCGI[0];
-	n->pipeOutput = pipesServer[1];
+	cgi->pid        = pid;
+	cgi->pipeInput  = pipesCGI[0];
+	cgi->pipeOutput = pipesServer[1];
 
 	if (conn->post != NULL) {
 		/* If we have a POST data to send, sets out output non-blocking 
 		 */
-		fcntl (n->pipeOutput, F_SETFL, O_NONBLOCK);
+		fcntl (cgi->pipeOutput, F_SETFL, O_NONBLOCK);
 
 		/* Start to send data 
 		 */
-		_send_post_data(n);
+		_send_post_data(cgi);
 	} else {
 		/* We don't need to send anything to the client,
 		 * so we close the pipe
 		 */
-		close(n->pipeOutput);
-		n->pipeOutput = -1;
+		close(cgi->pipeOutput);
+		cgi->pipeOutput = -1;
 	}
 
 	/* TODO Yeah, I know, this is not a perfect solution, but it is very
 	 * dificult do this in cherokee
 	 */
-	cherokee_buffer_new (&n->data);
-	cherokee_buffer_ensure_size (n->data, 2 * 1024); /* 2 kb for headers */
+	cherokee_buffer_new (&cgi->data);
+	cherokee_buffer_ensure_size (cgi->data, 2 * 1024); /* 2 kb for headers */
 
 	return ret_ok;
 }
 
 
-ret_t
-cherokee_handler_cgi_free (cherokee_handler_cgi_t *n)
+static int
+do_reap (void)
 {
+	pid_t pid;
+	int   child_count;
+	int   status;
+
+	/* Reap defunct children until there aren't any more. 
+	 */
+	for (child_count = 0; ; ++child_count)
+        {
+		pid = waitpid (-1, &status, WNOHANG);
+
+                /* none left */
+		if (pid == 0) break;
+
+		else if (pid < 0) {
+			/* because of ptrace */
+			if (errno == EINTR) continue;
+			break;
+		}
+        }
+
+	return child_count;
+}
+
+
+ret_t
+cherokee_handler_cgi_free (cherokee_handler_cgi_t *cgi)
+{
+	pid_t pid_status;
+	int   status = 0;
+
 	_LOG("CGI: closing\n\n");
 
-	close(n->pipeInput);
-	close(n->pipeOutput);
+	/* Close the connection with the CGI
+	 */
+	close(cgi->pipeInput);
+	close(cgi->pipeOutput);
 
-	if(n->data) {
-		cherokee_buffer_free (n->data);
+        /* Maybe kill the CGI
+	 */
+#if 0
+	if (cgi->pid > 0) {
+		pid_status = waitpid (cgi->pid, &status, WNOHANG);
+		if (pid_status <= 0) {
+			kill (cgi->pid, SIGTERM);
+		}
+	}
+#endif
+
+	/* Free the rest of the handler CGI memory
+	 */
+	if(cgi->data) {
+		cherokee_buffer_free (cgi->data);
 	}
 
-	if(n->pathInfo) {
-		cherokee_buffer_free (n->pathInfo);
+	if(cgi->pathInfo) {
+		cherokee_buffer_free (cgi->pathInfo);
 	}
 
-	if(n->fileName) {
-		cherokee_buffer_free (n->fileName);
+	if(cgi->fileName) {
+		cherokee_buffer_free (cgi->fileName);
 	}
 
-	if (n->parameter) {
-		cherokee_buffer_free (n->parameter);
+	if (cgi->parameter) {
+		cherokee_buffer_free (cgi->parameter);
 	}
 
-	free (n);
-	
+	free (cgi);
+
+	/* for some reason, we have seen that the SIGCHLD signal does not call to
+	 * our handler in a server with a lot of requests, so the wait() call,
+	 * necessary to free the resources used by the CGI, is not called. So I
+	 * think that a possible solution couble be to put the waitpid call in the
+	 * _free method of this handler, so when the handler ends, this will free
+	 * the resources used by our cool CGI.
+	 */
+	do_reap();
+
 	return ret_ok;
 }
 
 
 ret_t
-_read_from_cgi (cherokee_handler_cgi_t *n, cherokee_buffer_t *buffer)
+_read_from_cgi (cherokee_handler_cgi_t *cgi, cherokee_buffer_t *buffer)
 {
 	int len, ret_poll;
-	struct pollfd polling = { n->pipeInput, POLLIN|POLLPRI, 0 };
+	struct pollfd polling = { cgi->pipeInput, POLLIN|POLLPRI, 0 };
 	CHEROKEE_TEMP(tmp,1024);
 
 	_LOG("CGI: Read from CGI... ");
@@ -547,7 +585,7 @@ _read_from_cgi (cherokee_handler_cgi_t *n, cherokee_buffer_t *buffer)
 
 	/* Read the data from the pipe:
 	 */
-	len = read (n->pipeInput, tmp, tmp_size);
+	len = read (cgi->pipeInput, tmp, tmp_size);
 	if (len > 0) {
 		cherokee_buffer_add (buffer, tmp, len);
 
@@ -608,27 +646,25 @@ cherokee_handler_cgi_add_headers (cherokee_handler_cgi_t *cgi, cherokee_buffer_t
 {
 	ret_t  ret;
 	char  *content;
-	char  *cont1, *cont2;
-	
+	int    end_len;
+
 	ret = _read_from_cgi (cgi, cgi->data);
 	if (ret != ret_ok) return ret;
 
-	cont1 = strstr(cgi->data->buf, "\n\n");
-	cont2 = strstr(cgi->data->buf, "\r\n\r\n");
-
-	content = min_string_not_null (cont1, cont2);
+	content = strstr(cgi->data->buf, "\n\n");
+	if (content != NULL) {
+		end_len = 2;
+	} else {
+		content = strstr(cgi->data->buf, CRLF CRLF);
+		end_len = 4;
+	}
 
 	if (content != NULL) {
 		int len = content - cgi->data->buf;
 		
 		cherokee_buffer_add (buffer, cgi->data->buf, len);
-		cherokee_buffer_add (buffer, CRLF, 2);
-
-		if (cont1 == content) {
-			cherokee_buffer_move_to_begin (cgi->data, len+2);
-		} else {
-			cherokee_buffer_move_to_begin (cgi->data, len+4);
-		}
+		cherokee_buffer_add (buffer, CRLF CRLF, 4);
+		cherokee_buffer_move_to_begin (cgi->data, len + end_len);
 
 		return ret_ok;
 	}
@@ -645,6 +681,7 @@ cherokee_handler_cgi_add_headers (cherokee_handler_cgi_t *cgi, cherokee_buffer_t
 
 static int _cgi_is_init = 0;
 
+#if 0
 #include <signal.h>
 
 static void 
@@ -653,6 +690,7 @@ child_finished(int sng)
 	int status;
 	while(waitpid (0, &status, WNOHANG) > 0);
 }
+#endif
 
 void
 cgi_init ()
@@ -662,5 +700,7 @@ cgi_init ()
 	}
 
 	_cgi_is_init = 1;
+#if 0
 	signal(SIGCHLD, child_finished);
+#endif
 }
