@@ -52,15 +52,14 @@ cherokee_handler_dirlist_new  (cherokee_handler_t **hdl, void *cnt, cherokee_tab
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	HANDLER(n)->connection  = cnt;
-	HANDLER(n)->support     = hsupport_nothing;
 
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_dirlist_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_dirlist_free;
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_dirlist_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_dirlist_free;
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_dirlist_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_dirlist_add_headers; 
 
-	*hdl = HANDLER(n);
+	HANDLER(n)->connection  = cnt;
+	HANDLER(n)->support     = hsupport_nothing;
 
 	/* Process the request_string, and build the arguments table..
 	 * We'll need it later
@@ -131,6 +130,7 @@ cherokee_handler_dirlist_new  (cherokee_handler_t **hdl, void *cnt, cherokee_tab
 	}
 
 
+	*hdl = HANDLER(n);
 	return ret_ok;
 }
 
