@@ -68,8 +68,8 @@ cherokee_handler_file_new  (cherokee_handler_t **hdl, cherokee_connection_t *cnt
 
 	HANDLER(n)->support = hsupport_length | hsupport_range;
 
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_file_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_file_free;
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_file_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_file_free;
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_file_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_file_add_headers;
 	
@@ -231,11 +231,13 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 			
 			/* Look for the mmaped content
 			 */
-			cherokee_mmap_get_default (&mmap);
-			ret = cherokee_mmap_get_fast (mmap, n->fd, &n->info, &info, &info_len);
-			if (ret == ret_ok) {
-				conn->mmaped     = info     + conn->range_start;
-				conn->mmaped_len = info_len - (conn->range_start + (info_len - conn->range_end));
+			ret = cherokee_mmap_get_default (&mmap);
+			if ((ret >= ret_ok) && (mmap)) {
+				ret = cherokee_mmap_get_fast (mmap, n->fd, &n->info, &info, &info_len);
+				if (ret == ret_ok) {
+					conn->mmaped     = info     + conn->range_start;
+					conn->mmaped_len = info_len - (conn->range_start + (info_len - conn->range_end));
+				}
 			}
 		}
 	}
@@ -245,11 +247,9 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 	if (conn->range_start != 0) {
 		conn->error_code = http_partial_content;
 
-		if (conn->mmaped != NULL) {
-			if (conn->range_start != 0) {
-				n->offset = conn->range_start;
-				lseek (n->fd, n->offset, SEEK_SET);
-			}
+		if (conn->mmaped == NULL) {
+			n->offset = conn->range_start;
+			lseek (n->fd, n->offset, SEEK_SET);
 		}
 	}
 
@@ -258,9 +258,9 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 	 */
 #if HAVE_SENDFILE
 	n->using_sendfile = ((conn->mmaped == NULL) &&
+			     (conn->encoder == NULL) &&
 			     (n->info.st_size >= 32768) && 
-			     (n->info.st_size < 2147483647) &&
-			     conn->encoder == NULL);
+			     (n->info.st_size < 2147483647));
 
 	if (n->using_sendfile) {
 		cherokee_connection_set_cork(conn, 1);
