@@ -36,22 +36,19 @@ cherokee_module_info_t cherokee_gnomevfs_info = {
 ret_t 
 cherokee_handler_gnomevfs_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties)
 {
-	cherokee_handler_gnomevfs_t *n = (cherokee_handler_gnomevfs_t *)malloc(sizeof(cherokee_handler_gnomevfs_t));
-	return_if_fail (n != NULL, ret_nomem);
+	CHEROKEE_NEW(n, handler_gnomevfs);
 
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	HANDLER(n)->connection  = cnt;
 
-	HANDLER(n)->support = hsupport_length | hsupport_range;
-
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_gnomevfs_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_gnomevfs_free;
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_gnomevfs_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_gnomevfs_free;
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_gnomevfs_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_gnomevfs_add_headers;
 
-	*hdl = HANDLER(n);
+	HANDLER(n)->connection  = cnt;
+	HANDLER(n)->support     = hsupport_length | hsupport_range;
 	
 	/* Init
 	 */
@@ -62,6 +59,9 @@ cherokee_handler_gnomevfs_new (cherokee_handler_t **hdl, void *cnt, cherokee_tab
 	n->openning    = open_none;
 	n->reading     = read_none;
 
+	/* Return the object
+	 */
+	*hdl = HANDLER(n);
 	return ret_ok;	
 }
 
