@@ -77,14 +77,14 @@ cherokee_handler_mono_new (cherokee_handler_t **hdl, cherokee_connection_t *cnt,
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	
-	HANDLER(n)->support = hsupport_nothing;
-	
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_mono_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_mono_free;
+		
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_mono_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_mono_free;
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_mono_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_mono_add_headers;
 
+	HANDLER(n)->support = hsupport_nothing;
+
 	/* Init
 	 */
 	n->socket      = -1;
