@@ -45,38 +45,38 @@ cherokee_module_info_t cherokee_nn_info = {
 static char *
 get_nearest_from_directory (char *directory, char *request)
 {
-	   char          *nearest  = NULL;
-	   int            min_diff = 9999;
-	   DIR           *dir;
-	   struct dirent *entry;
+	char          *nearest  = NULL;
+	int            min_diff = 9999;
+	DIR           *dir;
+	struct dirent *entry;
 
 
-	   dir = opendir(directory);
-	   if (dir == NULL) goto go_out;
+	dir = opendir(directory);
+	if (dir == NULL) goto go_out;
 
-	   while ((entry = readdir (dir)) != NULL)
-	   { 
-			 int dis;
+	while ((entry = readdir (dir)) != NULL)
+	{ 
+		int dis;
 			 
-			 if (!strncmp (entry->d_name, ".", 1)) continue;
-			 if (!strncmp (entry->d_name, "..",2)) continue;
+		if (!strncmp (entry->d_name, ".", 1)) continue;
+		if (!strncmp (entry->d_name, "..",2)) continue;
 
-			 dis = distance ((char *)request, entry->d_name);
-			 if (dis < min_diff) {
-				    min_diff = dis;
+		dis = distance ((char *)request, entry->d_name);
+		if (dis < min_diff) {
+			min_diff = dis;
 
-				    if (nearest != NULL) {
-					    free (nearest);
-				    }
+			if (nearest != NULL) {
+				free (nearest);
+			}
 
-				    nearest = strdup(entry->d_name);
-			 }
+			nearest = strdup(entry->d_name);
+		}
 			 
-	   }
-	   closedir (dir);
+	}
+	closedir (dir);
 
 go_out:
-	   return nearest;
+	return nearest;
 }
 
 
@@ -146,7 +146,7 @@ cherokee_handler_nn_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *
 	/* Create the redir handler
 	 */
 	ret = cherokee_handler_redir_new (hdl, cnt, properties);
-	HANDLER(*hdl)->init = (handler_func_init_t) cherokee_handler_nn_init;
+	MODULE(*hdl)->init = (handler_func_init_t) cherokee_handler_nn_init;
 	
 	return ret;
 }
@@ -163,7 +163,6 @@ cherokee_handler_nn_init (cherokee_handler_t *hdl)
 	 */
 	new_request = get_nearest (conn->local_directory, conn->request);
 	
-//	hdl->redirect = new_request;
 	cherokee_buffer_add (conn->redirect, new_request, strlen(new_request));
 	CONN(hdl->connection)->error_code = http_moved_permanently;
 	
@@ -179,16 +178,16 @@ static int _nn_is_init = 0;
 void
 nn_init ()
 {
-	   /* Is init?
-	    */
-	   if (_nn_is_init)
-			 return;
+	/* Is init?
+	 */
+	if (_nn_is_init)
+		return;
 	   
-	   /* Init the dependences
-	    */
-	   common_init();
+	/* Init the dependences
+	 */
+	common_init();
 
-	   _nn_is_init = 1;
+	_nn_is_init = 1;
 }
 
 
