@@ -50,7 +50,7 @@ cherokee_handler_phpcgi_new  (cherokee_handler_t **hdl, void *cnt, cherokee_tabl
 	   
 	/* Redefine the init method
 	 */
-	HANDLER(*hdl)->init = (handler_func_init_t) cherokee_handler_phpcgi_init;
+	MODULE(*hdl)->init = (handler_func_init_t) cherokee_handler_phpcgi_init;
 
 	return ret_ok;
 }
