@@ -41,14 +41,13 @@ cherokee_handler_redir_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	HANDLER(n)->connection  = cnt;
 
-	HANDLER(n)->support = 0;
-
-	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_redir_init;
-	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_redir_free;
-	HANDLER(n)->step        = NULL;
+	MODULE(n)->init         = (handler_func_init_t) cherokee_handler_redir_init;
+	MODULE(n)->free         = (handler_func_free_t) cherokee_handler_redir_free;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_redir_add_headers;
+
+	HANDLER(n)->connection  = cnt;
+	HANDLER(n)->support     = hsupport_nothing;
 	
 	if (properties) {
 		n->target_url = cherokee_table_get_val (properties, "url");
@@ -73,12 +72,8 @@ cherokee_handler_redir_init (cherokee_handler_redir_t *n)
 	request_end = (conn->request->len - conn->web_directory->len);
 	request_endding = conn->request->buf + conn->web_directory->len;
 
-//	HANDLER(n)->redirect = (char *) malloc (request_end + n->target_url_len +1);
 	cherokee_buffer_ensure_size (conn->redirect, request_end + n->target_url_len +1);
 
-//	memcpy (HANDLER(n)->redirect, n->target_url, n->target_url_len);
-//	memcpy (HANDLER(n)->redirect + n->target_url_len, request_endding, request_end);
-//	HANDLER(n)->redirect[request_end + n->target_url_len] = '\0';
 	cherokee_buffer_add (conn->redirect, n->target_url, n->target_url_len);
 	cherokee_buffer_add (conn->redirect, request_endding, request_end);
 
