@@ -352,14 +352,14 @@ cherokee_handler_server_info_new  (cherokee_handler_t **hdl, cherokee_connection
 	   /* Init the base class object
 	    */
 	   cherokee_handler_init_base(HANDLER(n), cnt);
-
-	   HANDLER(n)->support = hsupport_length | hsupport_range;
 	   
-	   HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_server_info_init;
-	   HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_server_info_free;
+	   MODULE(n)->init         = (handler_func_init_t) cherokee_handler_server_info_init;
+	   MODULE(n)->free         = (handler_func_free_t) cherokee_handler_server_info_free;
 	   HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_server_info_step;
 	   HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_server_info_add_headers;
 
+	   HANDLER(n)->support = hsupport_length | hsupport_range;
+
 	   /* Init
 	    */
 	   cherokee_buffer_new (&n->buffer);
