@@ -46,7 +46,7 @@ const struct {
 	int   len;
 } 
 known_headers_names [] = {
-	{"Accept",           5},
+	{"Accept",           6},
 	{"Host",             4},
 	{"Range",            5},
 	{"Accept-Encoding", 15},
@@ -394,12 +394,25 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 		return ret_error;
 	}
 
+	/* Set the header buffer
+	 */
 	hdr->input_buffer = buffer;
 
 #ifdef HEADER_INTERNAL_DEBUG
 	hdr->input_buffer_crc = cherokee_buffer_crc32 (buffer);
 #endif
 
+	/* Look for the header endding
+	 */
+	header_end = strstr (buffer->buf, CRLF CRLF);
+	if (header_end == NULL) {
+		return ret_error;
+	}
+
+	header_end += 4;
+	hdr->input_header_len = (header_end - buffer->buf);
+
+
 	/* Parse the speacial first line
 	 */
 	switch (type) {
@@ -426,17 +439,6 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 	}
 
 
-	/* Look for the header endding
-	 */
-	header_end = strstr (buffer->buf, CRLF CRLF);
-	if (header_end == NULL) {
-		return ret_error;
-	}
-
-	header_end += 4;
-	hdr->input_header_len = (header_end - buffer->buf);
-
-
 	/* Parse the rest of headers
 	 */
 	while ((end = get_new_line(begin)) && (end < header_end))
@@ -741,6 +743,8 @@ cherokee_header_foreach (cherokee_header_t *hdr, cherokee_header_foreach_func_t
 			 cherokee_buffer_t *name, cherokee_buffer_t *cont, void *param)
 {
 	ret_t ret;
+	char *header_name;
+	int header_len;
 	int i;
 
 	/* Known headers
@@ -748,9 +752,9 @@ cherokee_header_foreach (cherokee_header_t *hdr, cherokee_header_foreach_func_t
 	for (i=0; i<HEADER_LENGTH; i++)
 	{
 		if (hdr->header[i].info_off != 0) {
-			char *header_name = known_headers_names[i].name;
-			int   header_len  = known_headers_names[i].len;
-			
+			header_name = known_headers_names[i].name;
+			header_len  = known_headers_names[i].len;
+
 			cherokee_buffer_make_empty (name);
 			cherokee_buffer_make_empty (cont);
 
@@ -764,12 +768,21 @@ cherokee_header_foreach (cherokee_header_t *hdr, cherokee_header_foreach_func_t
 
 	/* Unknown headers
 	 */
-	for (i=0; i<hdr->unknowns_len; i++) 
+	for (i=0; i < hdr->unknowns_len; i++)
 	{
-		char *points;
-		cherokee_header_unknown_entry_t *entry = &hdr->unknowns[i];
+		header_name = hdr->unknowns[i].header_off + hdr->input_buffer->buf;
+		header_len = index(header_name, ':') - header_name;
 
-//		points = index (entry->header_off
+		cherokee_buffer_make_empty (name);
+		cherokee_buffer_make_empty (cont);
+
+		cherokee_buffer_add(name, header_name, header_len);
+		cherokee_buffer_add(cont, 
+				hdr->unknowns[i].header_info_off + hdr->input_buffer->buf,
+				hdr->unknowns[i].header_info_len);
+
+		ret = func(name, cont, param);
+		if (ret < ret_ok) return ret;
 	}
 
 	return ret_ok;
