@@ -26,6 +26,9 @@
 #ifndef __HTTP_H__
 #define __HTTP_H__
 
+#include "common.h"
+#include "buffer.h"
+
 typedef enum {
 	   http_version_09,
 	   http_version_10,
@@ -65,7 +68,7 @@ typedef enum {
 #define http_accepted_string              "202 Accepted"
 #define http_partial_content_string       "206 Partial Content"
 #define http_moved_permanently_string     "301 Moved Permanently"
-#define http_moved_temporarily_string     "302 Moved Temporarily"
+#define http_moved_temporarily_string     "302 Found"
 #define http_not_modified_string          "304 Not Modified"
 #define http_bad_request_string           "400 Bad Request"
 #define http_unauthorized_string          "401 Authorization Required"
@@ -86,4 +89,8 @@ typedef enum {
 #define http_type_500(c)  ((c >= 500) && (c <= http_type_500_max))
 
 
+ret_t cherokee_http_method_to_string  (cherokee_http_method_t  method,  const char **str, int *str_len);
+ret_t cherokee_http_version_to_string (cherokee_http_version_t version, const char **str, int *str_len);
+ret_t cherokee_http_code_copy         (cherokee_http_t code, cherokee_buffer_t *buf);
+
 #endif /* __HTTP_H__ */
