@@ -160,7 +160,7 @@ extern int   yy_icons_lex(void);
 extern char *yy_icons_text;
 extern int   yy_icons_lineno;
 
-list_t  current_list;
+static list_t  current_list;
 
 
 void
