@@ -30,6 +30,7 @@
 #define __CHEROKEE_LOGGER_H__
 
 #include "common.h"
+#include "module.h"
 #include "buffer.h"
 #include "table.h"
 #include "http.h"
@@ -47,11 +48,12 @@ typedef ret_t (* logger_func_write_string_t) (void  *logger, const char *format)
 
 
 typedef struct {
+	/* Base
+	 */
+	cherokee_module_t module;
 
 	/* Pure virtual methods
 	 */
-	logger_func_init_t         init;
-	logger_func_free_t         free;
 	logger_func_flush_t        flush;
 	logger_func_write_access_t write_access;
 	logger_func_write_error_t  write_error;
