@@ -45,43 +45,46 @@
      T_HANDLER = 271,
      T_USER = 272,
      T_GROUP = 273,
-     T_SERVER = 274,
-     T_USERDIR = 275,
-     T_URL = 276,
-     T_PIDFILE = 277,
-     T_LISTEN = 278,
-     T_FILEDIR = 279,
-     T_HIDE_VERSION = 280,
-     T_HIDE_SERVER_NAME = 281,
-     T_ENCODER = 282,
-     T_ALLOW = 283,
-     T_BGCOLOR = 284,
-     T_TEXT = 285,
-     T_LINK = 286,
-     T_ALINK = 287,
-     T_VLINK = 288,
-     T_BACKGROUND = 289,
-     T_DIRECTORYINDEX = 290,
-     T_IPV6 = 291,
-     T_SHOW = 292,
-     T_CHROOT = 293,
-     T_HEADER_FILE = 294,
-     T_ICONS = 295,
-     T_AUTH = 296,
-     T_NAME = 297,
-     T_METHOD = 298,
-     T_PASSWDFILE = 299,
-     T_SSL_CA_LIST_FILE = 300,
-     T_FROM = 301,
-     T_SOCKET = 302,
-     T_NUMBER = 303,
-     T_PORT = 304,
-     T_QSTRING = 305,
-     T_FULLDIR = 306,
-     T_ID = 307,
-     T_HTTP_URL = 308,
-     T_HOSTNAME = 309,
-     T_IP = 310
+     T_POLICY = 274,
+     T_SERVER = 275,
+     T_USERDIR = 276,
+     T_URL = 277,
+     T_PIDFILE = 278,
+     T_LISTEN = 279,
+     T_FILEDIR = 280,
+     T_HIDE_VERSION = 281,
+     T_HIDE_SERVER_NAME = 282,
+     T_ENCODER = 283,
+     T_ALLOW = 284,
+     T_BGCOLOR = 285,
+     T_TEXT = 286,
+     T_LINK = 287,
+     T_ALINK = 288,
+     T_VLINK = 289,
+     T_BACKGROUND = 290,
+     T_DIRECTORYINDEX = 291,
+     T_IPV6 = 292,
+     T_SHOW = 293,
+     T_CHROOT = 294,
+     T_HEADER_FILE = 295,
+     T_ICONS = 296,
+     T_AUTH = 297,
+     T_NAME = 298,
+     T_METHOD = 299,
+     T_PASSWDFILE = 300,
+     T_SSL_CA_LIST_FILE = 301,
+     T_FROM = 302,
+     T_SOCKET = 303,
+     T_LOG_FLUSH_INTERVAL = 304,
+     T_INCLUDE = 305,
+     T_NUMBER = 306,
+     T_PORT = 307,
+     T_QSTRING = 308,
+     T_FULLDIR = 309,
+     T_ID = 310,
+     T_HTTP_URL = 311,
+     T_HOSTNAME = 312,
+     T_IP = 313
    };
 #endif
 #define T_QUOTE 258
@@ -100,43 +103,46 @@
 #define T_HANDLER 271
 #define T_USER 272
 #define T_GROUP 273
-#define T_SERVER 274
-#define T_USERDIR 275
-#define T_URL 276
-#define T_PIDFILE 277
-#define T_LISTEN 278
-#define T_FILEDIR 279
-#define T_HIDE_VERSION 280
-#define T_HIDE_SERVER_NAME 281
-#define T_ENCODER 282
-#define T_ALLOW 283
-#define T_BGCOLOR 284
-#define T_TEXT 285
-#define T_LINK 286
-#define T_ALINK 287
-#define T_VLINK 288
-#define T_BACKGROUND 289
-#define T_DIRECTORYINDEX 290
-#define T_IPV6 291
-#define T_SHOW 292
-#define T_CHROOT 293
-#define T_HEADER_FILE 294
-#define T_ICONS 295
-#define T_AUTH 296
-#define T_NAME 297
-#define T_METHOD 298
-#define T_PASSWDFILE 299
-#define T_SSL_CA_LIST_FILE 300
-#define T_FROM 301
-#define T_SOCKET 302
-#define T_NUMBER 303
-#define T_PORT 304
-#define T_QSTRING 305
-#define T_FULLDIR 306
-#define T_ID 307
-#define T_HTTP_URL 308
-#define T_HOSTNAME 309
-#define T_IP 310
+#define T_POLICY 274
+#define T_SERVER 275
+#define T_USERDIR 276
+#define T_URL 277
+#define T_PIDFILE 278
+#define T_LISTEN 279
+#define T_FILEDIR 280
+#define T_HIDE_VERSION 281
+#define T_HIDE_SERVER_NAME 282
+#define T_ENCODER 283
+#define T_ALLOW 284
+#define T_BGCOLOR 285
+#define T_TEXT 286
+#define T_LINK 287
+#define T_ALINK 288
+#define T_VLINK 289
+#define T_BACKGROUND 290
+#define T_DIRECTORYINDEX 291
+#define T_IPV6 292
+#define T_SHOW 293
+#define T_CHROOT 294
+#define T_HEADER_FILE 295
+#define T_ICONS 296
+#define T_AUTH 297
+#define T_NAME 298
+#define T_METHOD 299
+#define T_PASSWDFILE 300
+#define T_SSL_CA_LIST_FILE 301
+#define T_FROM 302
+#define T_SOCKET 303
+#define T_LOG_FLUSH_INTERVAL 304
+#define T_INCLUDE 305
+#define T_NUMBER 306
+#define T_PORT 307
+#define T_QSTRING 308
+#define T_FULLDIR 309
+#define T_ID 310
+#define T_HTTP_URL 311
+#define T_HOSTNAME 312
+#define T_IP 313
 
 
 
@@ -156,7 +162,7 @@ typedef union YYSTYPE {
 	   void *list;
 } YYSTYPE;
 /* Line 1240 of yacc.c.  */
-#line 160 "read_config_grammar.h"
+#line 166 "read_config_grammar.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
