@@ -34,6 +34,7 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <sys/wait.h>
+#include <signal.h>
 
 #include "server.h"
 #include "socket.h"
@@ -41,10 +42,9 @@
 #include "log.h"
 #include "ncpus.h"
 #include "mime.h"
-
-#ifdef HAVE_PTHREAD
 #include "thread.h"
-#endif
+#include "list_ext.h"
+
 
 static ret_t init_tls (void);
 
@@ -58,15 +58,22 @@ cherokee_server_new  (cherokee_server_t **srv)
 	 */
 	CHEROKEE_NEW_STRUCT(n, server);
 	
-	n->wanna_exit      = 0;
-	n->config_filename = NULL;
-	
+	n->wanna_exit  = 0;
+
 	/* Sockets
 	 */
 	n->socket     = -1;
 	n->socket_tls = -1;
 	n->ipv6       =  1;
 
+	/* Config files
+	 */
+	INIT_LIST_HEAD(&n->include_list);
+	n->config_file = NULL;	
+	n->icons_file  = NULL;
+	n->mime_file   = NULL;
+	
+
 	CHEROKEE_MUTEX_INIT (&n->accept_mutex, NULL);
 //	CHEROKEE_MUTEX_INIT (&n->socket_tls_mutex, NULL);
 
@@ -90,14 +97,12 @@ cherokee_server_new  (cherokee_server_t **srv)
 	n->userdir_handler= NULL;
 	n->start_time     = time(NULL);
 
-	n->icons_file     = NULL;
-	n->mime_file      = NULL;
-
 	n->keepalive      = true;
 	n->keepalive_max  = MAX_KEEPALIVE;
 
 	n->ncpus          = -1;
 	n->thread_num     = 0;
+	n->thread_policy  = -1;
 
 	n->chroot         = NULL;
 	n->chrooted       = 0;
@@ -153,6 +158,14 @@ cherokee_server_new  (cherokee_server_t **srv)
 		return ret;
 	}
 
+	/* Get system fd limimt..
+	 */
+	ret = get_system_fd_num_limit (&n->system_fd_limit);
+	if (ret < ret_ok) {
+		PRINT_ERROR ("Can not get file descriptor limit\n");
+		return ret;
+	}
+
 	/* Return the object
 	 */
 	*srv = n;
@@ -283,7 +296,7 @@ initialize_server_socket4 (cherokee_server_t *srv, unsigned short port, int *srv
 {
 	int re;
 	int srv_socket;
-	struct sockaddr_in  sockaddr;
+	struct sockaddr_in sockaddr;
 	
 	srv_socket = socket (AF_INET, SOCK_STREAM, 0);
 	if (srv_socket <= 0) {
@@ -296,6 +309,8 @@ initialize_server_socket4 (cherokee_server_t *srv, unsigned short port, int *srv
 
 	/* Bind the socket
 	 */
+	memset (&sockaddr, 0, sizeof(struct sockaddr_in));
+
 	sockaddr.sin_port   = htons (port);
 	sockaddr.sin_family = AF_INET;
 
@@ -335,12 +350,13 @@ initialize_server_socket6 (cherokee_server_t *srv, unsigned short port, int *srv
 		return ret_error;
 	}
 
-
 	*srv_socket_ret = srv_socket;
 	set_server_socket_opts(srv_socket);
 
 	/* Bind the socket
 	 */
+	memset (&sockaddr, 0, sizeof(struct sockaddr_in6));
+
 	sockaddr.sin6_port   = htons (port);  /* Transport layer port #   */
 	sockaddr.sin6_family = AF_INET6;      /* AF_INET6                 */
 
@@ -361,15 +377,11 @@ print_banner (cherokee_server_t *srv)
 {
 	char *method;
 	CHEROKEE_NEW(n,buffer);
-	
-	
+
+	/* First line
+	 */
 	cherokee_buffer_add_va (n, "Cherokee Web Server %s: ", VERSION);
 	cherokee_buffer_add_va (n, "Listening on port %d", srv->port);
-	if (srv->thread_num < 1) {
-		cherokee_buffer_add (n, ", single thread", 15);
-	} else {
-		cherokee_buffer_add_va (n, ", %d threads", srv->thread_num+1);
-	}
 	
 	cherokee_fdpoll_get_method (srv->main_thread->fdpoll, &method);
 	cherokee_buffer_add_va (n, ", using %s", method);
@@ -378,6 +390,33 @@ print_banner (cherokee_server_t *srv)
 		cherokee_buffer_add (n, ", chrooted", 10);
 	}
 
+	cherokee_buffer_add_va (n, ", %d fds limit", srv->system_fd_limit);
+
+	cherokee_buffer_add (n, "\n", 1);
+
+	/* Second line
+	 */
+	if (srv->thread_num < 1) {
+		cherokee_buffer_add (n, "single thread", 13);
+	} else {
+		cherokee_buffer_add_va (n, "%d threads", srv->thread_num+1);
+		cherokee_buffer_add_va (n, ", %d fds in each", srv->system_fd_limit / (srv->thread_num+1));	
+
+		switch (srv->thread_policy) {
+		case SCHED_FIFO:
+			cherokee_buffer_add (n, ", FIFO scheduling policy", 24);
+			break;
+		case SCHED_RR:
+			cherokee_buffer_add (n, ", RR scheduling policy", 22);
+			break;
+		default:
+			cherokee_buffer_add (n, ", standard scheduling policy", 28);
+			break;
+		}
+	}
+
+	/* Print it!
+	 */
 	printf ("%s\n", n->buf);
 	cherokee_buffer_free (n);
 	
@@ -388,21 +427,20 @@ print_banner (cherokee_server_t *srv)
 static ret_t
 initialize_server_socket (cherokee_server_t *srv, unsigned short port, int *srv_socket_ptr)
 {
-	ret_t ret;
 	int   flags;
 	int   srv_socket;
+	ret_t ret = ret_error;
 
 #ifdef HAVE_IPV6
 	if (srv->ipv6) {
 		ret = initialize_server_socket6 (srv, port, srv_socket_ptr);
-	} 
+	}
 #endif
-	if ((ret != ret_ok) || (srv->ipv6 == 0)) {
+
+	if ((srv->ipv6 == 0) || (ret != ret_ok)) {
 		ret = initialize_server_socket4 (srv, port, srv_socket_ptr);
 	}
 
-	srv_socket = *srv_socket_ptr;
-
 	if (ret != 0) {
 		uid_t uid = getuid();
 		gid_t gid = getgid();
@@ -411,6 +449,8 @@ initialize_server_socket (cherokee_server_t *srv, unsigned short port, int *srv_
 		return ret_error;
 	}
 	   
+	srv_socket = *srv_socket_ptr;
+
 	/* Set no-delay mode
 	 */
 	flags = fcntl (srv_socket, F_GETFL, 0);
@@ -437,26 +477,35 @@ static ret_t
 initialize_server_threads (cherokee_server_t *srv)
 {	
 	ret_t ret;
-	int   fd_limit, i;
-	int   fds_per_thread;
+	int   i, fds_per_thread;
 
 	/* Get the system fd number limit
 	 */
-	get_system_fd_num_limit (&fd_limit);
-	fds_per_thread = fd_limit / srv->ncpus;
+	fds_per_thread = srv->system_fd_limit / srv->ncpus;
 
 	/* Create the main thread
 	 */
-	ret = cherokee_thread_new (&srv->main_thread, srv, thread_sync, fds_per_thread);
+	ret = cherokee_thread_new (&srv->main_thread, srv, thread_sync, 
+				   srv->system_fd_limit, fds_per_thread);
 	if (ret < ret_ok) return ret;
 
-#ifdef HAVE_PTHREAD
-	/* Async threads starting
+	/* If Cherokee is compiled in single thread mode, it has to
+	 * add the server socket to the fdpoll of the sync thread
 	 */
+#ifndef HAVE_PTHREAD
+	ret = cherokee_fdpoll_add (srv->main_thread->fdpoll, srv->socket, 0);
+	if (ret < ret_ok) return ret;
+#endif
+
+	/* If Cherokee is compiles in multi-thread mode, it has to
+	 * launch the threads
+	 */
+#ifdef HAVE_PTHREAD
 	for (i=0; i<srv->ncpus - 1; i++) {
 		cherokee_thread_t *thread;
 
-		ret = cherokee_thread_new (&thread, srv, thread_async, fds_per_thread);
+		ret = cherokee_thread_new (&thread, srv, thread_async, 
+					   srv->system_fd_limit, fds_per_thread);
 		if (ret < ret_ok) return ret;
 		
 		list_add ((list_t *)thread, &srv->thread_list);
@@ -504,15 +553,12 @@ cherokee_server_init (cherokee_server_t *srv)
 	cherokee_table_foreach (srv->vservers_ref, for_each_vserver_init_tls_func);
 #endif
 
-        /* Threads / CPUs
+        /* Maybe recalculate the thread number
 	 */
 	if (srv->ncpus == -1) {
 		dcc_ncpus (&srv->ncpus);
 	}
-	
-	ret = initialize_server_threads (srv);
-	if (ret < ret_ok) return ret;
-	
+		
 	/* Chroot
 	 */
 	if (srv->chroot) {
@@ -529,10 +575,16 @@ cherokee_server_init (cherokee_server_t *srv)
 		return ret;
 	}
 
+	/* Change current directory
+	 */
 	chdir ("/");
 
-	print_banner (srv);
+	/* Create the threads
+	 */
+	ret = initialize_server_threads (srv);
+	if (ret < ret_ok) return ret;
 
+	print_banner (srv);
 	return ret_ok;
 }
 
@@ -556,6 +608,10 @@ for_each_func_free_vserver (const char *key, void *vserver)
 	cherokee_virtual_server_free (vserver);
 }
 
+static void
+free_include_list (cherokee_server_t *srv)
+{
+}
 
 ret_t 
 cherokee_server_clean (cherokee_server_t *srv)
@@ -586,6 +642,11 @@ cherokee_server_clean (cherokee_server_t *srv)
 		srv->chroot = NULL;
 	}
 
+
+	/* Clean config files entries
+	 */
+	cherokee_list_free (&srv->include_list, free);
+
 	if (srv->icons_file != NULL) {
 		free (srv->icons_file);
 		srv->icons_file = NULL;
@@ -615,7 +676,7 @@ cherokee_server_clean (cherokee_server_t *srv)
 
 	/* Clean the virtual servers table
 	 */	
-	cherokee_list_free (srv->vservers, cherokee_virtual_server_free);
+	cherokee_list_free (&srv->vservers, cherokee_virtual_server_free);
 	cherokee_table_clean (srv->vservers_ref);
 
 	/* Icons 
@@ -662,7 +723,7 @@ cherokee_server_free (cherokee_server_t *srv)
 	cherokee_virtual_server_free (srv->vserver_default);
 	srv->vserver_default = NULL;
 
-	cherokee_list_free (srv->vservers, cherokee_virtual_server_free);
+	cherokee_list_free (&srv->vservers, cherokee_virtual_server_free);
 	cherokee_table_free (srv->vservers_ref);
 
 	cherokee_module_loader_free (srv->loader);
@@ -692,9 +753,13 @@ cherokee_server_free (cherokee_server_t *srv)
 		srv->chroot = NULL;
 	}
 
-	if (srv->config_filename != NULL) {
-		free (srv->config_filename);
-		srv->config_filename = NULL;
+	/* Clean config files entries
+	 */
+	cherokee_list_free (&srv->include_list, free);
+
+	if (srv->config_file != NULL) {
+		free (srv->config_file);
+		srv->config_file = NULL;
 	}
 
 	if (srv->icons_file != NULL) {
@@ -707,8 +772,7 @@ cherokee_server_free (cherokee_server_t *srv)
 		   srv->mime_file = NULL;
 	}
 
-	free (srv);
-	
+	free (srv);	
 	return ret_ok;
 }
 
@@ -796,7 +860,7 @@ cherokee_server_step (cherokee_server_t *srv)
 	/* Get the time
 	 */
 	update_bogo_now (srv);
-	ret = cherokee_thread_step (srv->main_thread);
+	ret = cherokee_thread_step (srv->main_thread, true);
 
 	/* Logger flush 
 	 */
@@ -815,10 +879,9 @@ cherokee_server_step (cherokee_server_t *srv)
 }
 
 
-ret_t 
-cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
+static ret_t
+read_config_file (cherokee_server_t *srv, char *filename)
 {
-	ret_t ret;
 	int   error;
 	void *bufstate;
 	extern FILE *yyin;
@@ -831,18 +894,18 @@ cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
 	/* Maybe set a default configuration filename
 	 */
 	if (filename == NULL) {
-		filename = (srv->config_filename)? srv->config_filename : CHEROKEE_CONFDIR"/cherokee.conf";
+		filename = (srv->config_file)? srv->config_file : CHEROKEE_CONFDIR"/cherokee.conf";
 
 	} else {
 		/* Maybe free an old filename
 		 */
-		if (srv->config_filename != NULL) {
-			free (srv->config_filename);
+		if (srv->config_file != NULL) {
+			free (srv->config_file);
 		}
 		
 		/* Store last configuration filename
 		 */
-		srv->config_filename = strdup(filename);
+		srv->config_file = strdup(filename);
 	}
 
 	/* Set the file to read
@@ -864,9 +927,26 @@ cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
 
 	fclose (yyin);
 
-	ret = (error == 0) ? ret_ok : ret_error;
-	if (ret < ret_ok) {
-		return ret;
+	return (error == 0) ? ret_ok : ret_error;
+}
+
+
+ret_t 
+cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
+{
+	ret_t   ret;
+	list_t *i;
+
+	/* Read the main config file
+	 */
+	ret = read_config_file (srv, filename);
+	if (ret < ret_ok) return ret;
+
+	/* Process the includes form the config file
+	 */
+	list_for_each (i, &srv->include_list) {
+		ret = read_config_file (srv, LIST_ITEM_INFO(i));
+		if (ret < ret_ok) return ret;
 	}
 
 	/* Maybe read the Icons file
@@ -967,7 +1047,7 @@ static void
 exit_parent (int sig)
 {
         /* parent simply exits when child says go... 
-	    */
+	 */
         exit(0);
 }
 
