@@ -56,7 +56,6 @@ typedef struct {
 	cherokee_buffer_t *bogo_now_string;
 
 	int    wanna_exit;
-	char  *config_filename;
 	
 	/* Virtual servers
 	 */
@@ -66,8 +65,10 @@ typedef struct {
 	
 	/* Threads
 	 */
+	uint32_t           system_fd_limit;
 	cherokee_thread_t *main_thread;
 	struct list_head   thread_list;
+	int                thread_policy;
 
 	/* Modules
 	 */
@@ -88,6 +89,7 @@ typedef struct {
 	int    log_flush_elapse;
 	time_t log_flush_next;
 
+
 	/* Main socket
 	 */
 	int   socket;
@@ -125,12 +127,17 @@ typedef struct {
 	char  *chroot;
 	int    chrooted;
 
-	char  *mime_file;
-	char  *icons_file;
-
 	char                           *userdir;              /* Eg: public_html */
 	cherokee_handler_table_entry_t *userdir_handler;     
 
+
+	/* Another config files
+	 */
+	char             *config_file;
+	char             *mime_file;
+	char             *icons_file;
+	struct list_head  include_list;
+
 } cherokee_server_t;
 
 
