@@ -70,7 +70,7 @@ thread_routine (void *data)
 	set_sigpipe();	
 
         while (thread->exit == 0) {
-                cherokee_thread_step (thread);
+                cherokee_thread_step (thread, false);
         }
 
         return NULL;
@@ -79,7 +79,8 @@ thread_routine (void *data)
 
 
 ret_t 
-cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_type_t type, int fd_num)
+cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_type_t type, 
+		      int system_fd_num, int fd_num)
 {
 	cherokee_server_t *srv = SRV(server);
 	CHEROKEE_NEW_STRUCT (n, thread);
@@ -90,7 +91,7 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_typ
 	INIT_LIST_HEAD((list_t*)&n->active_list);
 	INIT_LIST_HEAD((list_t*)&n->reuse_list);
 	
-	cherokee_fdpoll_new (&n->fdpoll, fd_num);
+	cherokee_fdpoll_new (&n->fdpoll, system_fd_num, fd_num);
 
 	n->exit              = 0;
 	n->active_list_num   = 0;
@@ -98,6 +99,11 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_typ
 	n->server            = server;
 	n->thread_type       = type;
 
+	/* Accepting information
+	 */
+	n->accept.recalculate    = 0;
+	n->accept.continuous     = 0;
+	n->accept.continuous_max = 0;
 
 	/* Return the object
 	 */
@@ -106,21 +112,26 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_typ
 	
 	/* Do some related work..
 	 */
-	switch (type) {
-	case thread_async:
+	if (type == thread_async) {
 #ifdef HAVE_PTHREAD
-		pthread_create (&n->thread, NULL, thread_routine, n);
+		pthread_attr_t attr;
+		struct sched_param param;
+
+		/* Init the thread attributes
+		 */
+		pthread_attr_init (&attr);
+		pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+		
+		/* Maybe set the scheduling policy
+		 */
+		if (srv->thread_policy != -1) {
+			pthread_attr_setschedpolicy (&attr, srv->thread_policy);
+		}
+
+		pthread_create (&n->thread, &attr, thread_routine, n);
 #else
 		SHOULDNT_HAPPEN;
 #endif		
-		break;
-
-	case thread_sync:
-		cherokee_fdpoll_add   (n->fdpoll, srv->socket, 0);
-		break;
-
-	default:
-		SHOULDNT_HAPPEN;
 	}
 	
 	return ret_ok;
@@ -332,8 +343,10 @@ process_active_connections (cherokee_thread_t *thd)
 			/* Maybe the buffer has a request (previous pipelined)
 			 */
 			if (! cherokee_buffer_is_empty (conn->incoming_header)) {
-				if (cherokee_connection_eoh (conn, conn->incoming_header, 
-							     conn->incoming_header->len)) {
+				if (cherokee_connection_eoh (conn, 
+							     conn->incoming_header, 
+							     conn->incoming_header->len)) 
+				{
 					goto phase_reading_header_EXIT;
 				}
 			}
@@ -491,23 +504,31 @@ process_active_connections (cherokee_thread_t *thd)
 			}
 
 		case phase_add_headers:
-			ret = cherokee_connection_build_header (conn);
-			if (ret == ret_eagain) continue;
 
-			if (! http_type_200(conn->error_code)) {
+			/* If it is an error, maybe we will want to rewrite
+			 * the response code to redirect the error page
+			 */
+			if ((!http_type_200(conn->error_code)) &&
+			    cherokee_buffer_is_empty(conn->redirect)) 
+			{
 				char *url = NULL;
 
 				/* maybe has a custom ErrorPage redirection
 				 */
-				cherokee_virtual_server_get_error_url (CONN_VSRV(conn), 
-								       conn->error_code,
-								       &url);
+				ret = cherokee_virtual_server_get_error_url (CONN_VSRV(conn), 
+									     conn->error_code,
+									     &url);
 
-				if (url != NULL) {
+				if (ret == ret_ok) {
 					cherokee_buffer_add (conn->redirect, url, strlen(url));
-					conn->error_code = http_moved_permanently;
+					conn->error_code = http_moved_temporarily;
 				}
 			}
+
+			/* Build the header
+			 */
+			ret = cherokee_connection_build_header (conn);
+			if (ret == ret_eagain) continue;
 			
 			/* If it has mmaped content, go to next stage
 			 */		     
@@ -525,21 +546,21 @@ process_active_connections (cherokee_thread_t *thd)
 					continue;
 				}
 				
-				if (http_type_300(conn->error_code)) {
+/*				if (http_type_300(conn->error_code)) {
 					maybe_purge_closed_connection (thd, conn);
 					continue;
 
-				} else if (http_type_400(conn->error_code) ||
+			    } */ else if (http_type_400(conn->error_code) ||
 					   http_type_500(conn->error_code)) 
 				{
 					char *filename = NULL;
 
 					/* Has a custom ErrorPage
 					 */
-					cherokee_virtual_server_get_error_page (CONN_VSRV(conn), 
-										conn->error_code,
-										&filename);
-					if (filename != NULL) {
+					ret = cherokee_virtual_server_get_error_page (CONN_VSRV(conn), 
+										      conn->error_code,
+										      &filename);
+					if (ret == ret_ok) {
 						cherokee_connection_send_response_page_file (conn, filename);
 					} else {
 						cherokee_connection_send_response_page_hardcoded (conn);
@@ -686,6 +707,8 @@ __accept_from_server (cherokee_thread_t *thd, int srv_socket, int is_tls)
 	ret = cherokee_socket_accept (CONN_SOCK(new_conn), srv_socket);
 	if (ret < ret_ok) goto error;
 	
+//	printf ("%p - New connection socket=%d\n", thd, SOCKET_FD(CONN_SOCK(new_conn)));
+
 	/* Set the connection time-out
 	 */
 	new_conn->timeout = THREAD_SRV(thd)->bogo_now + THREAD_SRV(thd)->timeout;
@@ -698,7 +721,11 @@ __accept_from_server (cherokee_thread_t *thd, int srv_socket, int is_tls)
 	
 	/* Add the new connection to the thread
 	 */
-	cherokee_thread_add_connection (thd, new_conn);
+	ret = cherokee_thread_add_connection (thd, new_conn);
+	if (ret < ret_ok) {
+		goto error;
+	}
+
 	return 1;
 
 error:
@@ -707,8 +734,40 @@ error:
 }
 
 
+static int
+__should_accept_more_from_server (cherokee_thread_t *thd, int re)
+{
+	const uint32_t recalculate_steps = 10;
+
+	/* Got new connection
+	 */
+	if (re > 0) {
+		thd->accept.continuous++;
+
+		if (thd->accept.recalculate <= 0) {
+			thd->accept.continuous_max = thd->accept.continuous;
+			return 1;
+		}
+
+		if (thd->accept.continuous > thd->accept.continuous_max) {
+			thd->accept.continuous_max = thd->accept.continuous;
+			thd->accept.recalculate--;
+			return 0;
+		}
+		
+		return 1;
+	}
+
+	/* Failed to get a new connection
+	 */
+	thd->accept.continuous  = 0;
+	thd->accept.recalculate = recalculate_steps;
+	return 0;
+}
+
+
 ret_t 
-cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd)
+cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd, cherokee_boolean_t dont_block)
 {
 	ret_t ret;
 	int   re;
@@ -728,13 +787,16 @@ cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd)
 	re = cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 	if (re <= 0) return ret_ok;
 
-	re = __accept_from_server (thd, srv->socket, 0);
+	do {
+		re = __accept_from_server (thd, srv->socket, 0);
+	} while (__should_accept_more_from_server (thd, re));
+
 	return process_active_connections (thd);
 }
 
 
 ret_t 
-cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
+cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd, cherokee_boolean_t dont_block)
 {
 	ret_t ret;
 	int   re;
@@ -749,36 +811,53 @@ cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
 		thd->pending_conns_num = 0;
 	}
 
-
-	if (thd->active_list_num == 0) {
-		CHEROKEE_MUTEX_LOCK (&THREAD_SRV(thd)->accept_mutex);
-
-		cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
-		cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
-		cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
-
-		re = __accept_from_server (thd, srv->socket, 0);		
-
-		cherokee_fdpoll_del (thd->fdpoll, srv->socket);
-
-		CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
-	} else 
+	if (! cherokee_fdpoll_is_full(thd->fdpoll)) 
 	{
-		int unlocked;
-		
-		unlocked = CHEROKEE_MUTEX_TRY_LOCK (&THREAD_SRV(thd)->accept_mutex);
-		if (! unlocked) {
-			cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
+		if ((thd->active_list_num == 0) && (!dont_block)) 
+		{
+			CHEROKEE_MUTEX_LOCK (&THREAD_SRV(thd)->accept_mutex);
+
+			ret = cherokee_fdpoll_add (thd->fdpoll, srv->socket, 0);
+			if (ret < ret_ok) {
+				CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
+				return 0;
+			}
 			cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
 			cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 
-			re = __accept_from_server (thd, srv->socket, 0);		
+			do {
+				re = __accept_from_server (thd, srv->socket, 0);		
+			} while (__should_accept_more_from_server (thd, re) &&
+				 (!cherokee_fdpoll_is_full(thd->fdpoll)));
 
 			cherokee_fdpoll_del (thd->fdpoll, srv->socket);
-
 			CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
-		} else {
-			cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
+
+		} else 
+		{
+			int unlocked;
+		
+			unlocked = CHEROKEE_MUTEX_TRY_LOCK (&THREAD_SRV(thd)->accept_mutex);
+			if (! unlocked) {
+				ret = cherokee_fdpoll_add (thd->fdpoll, srv->socket, 0);
+				if (ret < ret_ok) {
+					CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
+					return 0;
+				}
+				cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
+				cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
+				
+				do {
+					re = __accept_from_server (thd, srv->socket, 0);
+				} while (__should_accept_more_from_server (thd, re) &&
+					 (!cherokee_fdpoll_is_full(thd->fdpoll)));
+				
+				cherokee_fdpoll_del (thd->fdpoll, srv->socket);
+				CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
+
+			} else {
+				cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
+			}
 		}
 	}
 
@@ -840,7 +919,11 @@ cherokee_thread_set_reusable_connection (cherokee_thread_t *thread, cherokee_con
 ret_t 
 cherokee_thread_add_connection (cherokee_thread_t *thd, cherokee_connection_t  *conn)
 {
-	cherokee_fdpoll_add (thd->fdpoll, SOCKET_FD(conn->socket), 0);
+	ret_t ret;
+
+	ret = cherokee_fdpoll_add (thd->fdpoll, SOCKET_FD(conn->socket), 0);
+	if (ret < ret_ok) return ret;
+
 	conn_set_mode (thd, conn, socket_reading);
 	add_connection (thd, conn);
 
