@@ -54,6 +54,13 @@ typedef struct {
 	list_t  reuse_list;     
 	int     pending_conns_num;      /* Waiting pipelining connections */
 
+	struct {
+		uint32_t continuous;
+		uint32_t continuous_max;
+		uint32_t recalculate;
+		
+	} accept;
+
 } cherokee_thread_t;
 
 #define THREAD(x)         ((cherokee_thread_t *)(x))
@@ -62,14 +69,14 @@ typedef struct {
 
 
 #ifdef HAVE_PTHREAD
-# define cherokee_thread_step(t) cherokee_thread_step_MULTI_THREAD(t)
-ret_t cherokee_thread_step_MULTI_THREAD (cherokee_thread_t  *thd);
+# define cherokee_thread_step(t,b) cherokee_thread_step_MULTI_THREAD(t,b)
+ret_t cherokee_thread_step_MULTI_THREAD  (cherokee_thread_t *thd, cherokee_boolean_t dont_block);
 #else
-# define cherokee_thread_step(t) cherokee_thread_step_SINGLE_THREAD(t)
-ret_t cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t  *thd);
+# define cherokee_thread_step(t,b) cherokee_thread_step_SINGLE_THREAD(t,b)
+ret_t cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd, cherokee_boolean_t dont_block);
 #endif
 
-ret_t cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_type_t type, int fd_num);
+ret_t cherokee_thread_new  (cherokee_thread_t **thd, void *server, cherokee_thread_type_t type, int system_fd_num, int fd_num);
 ret_t cherokee_thread_free (cherokee_thread_t  *thd);
 
 ret_t cherokee_thread_get_new_connection      (cherokee_thread_t *thd, cherokee_connection_t **conn);
