@@ -60,9 +60,9 @@ cherokee_validator_htpasswd_new (cherokee_validator_htpasswd_t **htpasswd, chero
 
 	/* Init 	
 	 */
-	MODULE(n)->func_init     = NULL;
-	MODULE(n)->func_free     = (module_func_free_t)     cherokee_validator_htpasswd_free;
-	VALIDATOR(n)->func_check = (validator_func_check_t) cherokee_validator_htpasswd_check;
+	cherokee_validator_init_base (VALIDATOR(n));
+	MODULE(n)->free     = (module_func_free_t)     cherokee_validator_htpasswd_free;
+	VALIDATOR(n)->check = (validator_func_check_t) cherokee_validator_htpasswd_check;
 	   
 	n->htpasswd_file_ref = NULL;
 
@@ -103,10 +103,10 @@ crypt_r_emu (const char *key, const char *salt)
 	char *ret;
 	char *tmp_crypt;
 
-	CHEROKEE_MUTEX_LOCK(__global_crypt_r_emu_mutex);
+	CHEROKEE_MUTEX_LOCK(&__global_crypt_r_emu_mutex);
 	tmp_crypt = crypt (key, salt);
 	ret = strdup (tmp_crypt);
-	CHEROKEE_MUTEX_UNLOCK(__global_crypt_r_emu_mutex);	
+	CHEROKEE_MUTEX_UNLOCK(&__global_crypt_r_emu_mutex);	
 
 	return ret;
 }
