@@ -35,30 +35,31 @@ cherokee_module_info_t cherokee_plain_info = {
 ret_t 
 cherokee_validator_plain_new (cherokee_validator_plain_t **plain, cherokee_table_t *properties)
 {
-	   CHEROKEE_NEW_STRUCT(n,validator_plain);
+	CHEROKEE_NEW_STRUCT(n,validator_plain);
 
-	   /* Init 		
-	    */
-	   MODULE(n)->func_init     = NULL;
-	   MODULE(n)->func_free     = (module_func_free_t)     cherokee_validator_plain_free;
-	   VALIDATOR(n)->func_check = (validator_func_check_t) cherokee_validator_plain_check;
+	/* Init 		
+	 */
+	cherokee_validator_init_base (VALIDATOR(n));
+
+	MODULE(n)->free     = (module_func_free_t)     cherokee_validator_plain_free;
+	VALIDATOR(n)->check = (validator_func_check_t) cherokee_validator_plain_check;
 	   
-	   n->plain_file_ref = NULL;
+	n->plain_file_ref = NULL;
 
-	   /* Get the properties
-	    */
-	   if (properties) {
-			 ret_t ret;
+	/* Get the properties
+	 */
+	if (properties) {
+		ret_t ret;
 			 
-			 ret = cherokee_table_get (properties, "file", &n->plain_file_ref);
-			 if (ret < ret_ok) {
-				    PRINT_ERROR ("plain validator needs a \"File\" property\n");
-				    return ret_error;
-			 }
-	   }
+		ret = cherokee_table_get (properties, "file", &n->plain_file_ref);
+		if (ret < ret_ok) {
+			PRINT_ERROR ("plain validator needs a \"File\" property\n");
+			return ret_error;
+		}
+	}
 	   
-	   *plain = n;
-	   return ret_ok;
+	*plain = n;
+	return ret_ok;
 }
 
 
@@ -73,56 +74,56 @@ cherokee_validator_plain_free (cherokee_validator_plain_t *plain)
 ret_t 
 cherokee_validator_plain_check (cherokee_validator_plain_t *plain, cherokee_connection_t *conn)
 {
-	   FILE  *f;
-	   ret_t  ret;
-	   int    len;
-	   char  *pass;
-	   CHEROKEE_TEMP(line, 128);
+	FILE  *f;
+	ret_t  ret;
+	int    len;
+	char  *pass;
+	CHEROKEE_TEMP(line, 128);
 
 
-	   f = fopen (plain->plain_file_ref, "r");
-	   if (f == NULL) {
-			 return ret_error;
-	   }
+	f = fopen (plain->plain_file_ref, "r");
+	if (f == NULL) {
+		return ret_error;
+	}
 
-	   ret = ret_error;
-	   while (!feof(f)) {
-			 if (fgets (line, line_size, f) == NULL)
-				    continue;
+	ret = ret_error;
+	while (!feof(f)) {
+		if (fgets (line, line_size, f) == NULL)
+			continue;
 
-			 len = strlen (line);
+		len = strlen (line);
 
-			 if (len <= 0) 
-				    continue;
+		if (len <= 0) 
+			continue;
 			 
-			 if (line[0] == '#')
-				    continue;
+		if (line[0] == '#')
+			continue;
 
-			 if (line[len-1] == '\n') 
-				    line[len-1] = '\0';
+		if (line[len-1] == '\n') 
+			line[len-1] = '\0';
 			 
-			 /* Split into user and encrypted password. 
-			  */
-			 pass = strchr (line, ':');
-			 if (pass == NULL) continue;
-			 *pass++ = '\0';
+		/* Split into user and encrypted password. 
+		 */
+		pass = strchr (line, ':');
+		if (pass == NULL) continue;
+		*pass++ = '\0';
 			 
-			 /* Is this the right user? 
-			  */
-			 if (strcmp (conn->user->buf, line) != 0) {
-				    continue;
-			 }
+		/* Is this the right user? 
+		 */
+		if (strcmp (conn->user->buf, line) != 0) {
+			continue;
+		}
 		
-			 /* Check the password
-			  */
-			 if (strcmp (conn->passwd->buf, pass) == 0) {
-				    ret = ret_ok;
-				    break;
-			 }
-	   }
+		/* Check the password
+		 */
+		if (strcmp (conn->passwd->buf, pass) == 0) {
+			ret = ret_ok;
+			break;
+		}
+	}
 
-	   fclose(f);
+	fclose(f);
 
-	   return ret;
+	return ret;
 }
 
